@@ -227,6 +227,8 @@ class Response:
                     ) for k, v in self.output.items() if k != "status" and (v or v is False)
                 ]
             )
+        else:
+            r = f"  {self.output}"
 
         # sanitize sensitive data for demos
         if config.sanitize and config.sanitize_file.is_file():
@@ -364,6 +366,7 @@ class Session():
         self.updated_at = time.monotonic()
         self.rl_log = [f"{self.updated_at - INIT_TS:.2f} [INIT] {type(self).__name__} object at {hex(id(self))}"]
         self.BatchRequest = BatchRequest
+        self.running_spinners = []
 
     @property
     def aio_session(self):
@@ -421,18 +424,19 @@ class Session():
             try:
                 req_log = LoggedRequests(_url.path_qs, method)
 
-                # -- // THE API REQUEST \\ --
-                #  -- // RATE LIMIT TEST \\ --
-                # await self.wait_for_token()
                 _start = time.monotonic()
                 now = time.monotonic() - INIT_TS
+
                 _try_cnt = [u.url for u in self.requests].count(_url.path_qs) + 1
                 self.rl_log += [
                     f'{now:.2f} [{method}]{_url.path_qs} Try: {_try_cnt}'
                 ]
-                #  -- // RATE LIMIT TEST \\ --
+
+                # TODO spinner steps on each other during long running requests
+                # need to check store prev msg when updating then restore it if that thread is still running
                 self.spinner.start(spin_txt_run)
-                self.req_cnt += 1
+                self.running_spinners += [spin_txt_run]
+                self.req_cnt += 1  # TODO may have deprecated now that logging requests
                 # TODO move batch_request _batch_request, get, put, etc into Session
                 # change where client is instantiated to _request / _batch_requests pass in the client
                 # remove aio_session property call ClientSession() direct
@@ -470,7 +474,13 @@ class Session():
 
             fail_msg = spin_txt_fail if self.silent else f"{spin_txt_fail}\n  {resp.output}"
             if not resp:
+                # TODO handle the following... pause/retry
+                # Request 3 [POST: /platform/licensing/v1/subscriptions/unassign]
+                #     Response:
+                #     status code: 503
+                #     upstream connect error or disconnect/reset before headers. reset reason: connection termination
                 self.spinner.fail(fail_msg)
+                self.running_spinners = [s for s in self.running_spinners if s != spin_txt_run]
                 if "invalid_token" in resp.output:
                     spin_txt_retry =  "(retry after token refresh)"
                     self.refresh_token()
@@ -489,7 +499,12 @@ class Session():
                     self.rl_log += [
                         f"{time.monotonic() - INIT_TS:.2f} [[bright_green]{resp.error}[/]] p/s: {resp.rl.remain_sec} {_url.path_qs}"
                     ]
-                self.spinner.stop()
+                # This handles long running API calls where subsequent calls finish before the previous...
+                self.running_spinners = [s for s in self.running_spinners if s != spin_txt_run]
+                if self.running_spinners:
+                    self.spinner.text = self.running_spinners[-1]
+                else:
+                    self.spinner.stop()
                 break
 
         return resp
@@ -688,7 +703,7 @@ class Session():
         log.debug("PAUSE {_pause:.2f}s...")
         time.sleep(_pause)
 
-    async def _batch_request(self, api_calls: List[BatchRequest],) -> List[Response]:
+    async def _batch_request(self, api_calls: List[BatchRequest], continue_on_fail: bool = False,) -> List[Response]:
         self.silent = True
         m_resp = []
         _tot_start = time.perf_counter()
@@ -698,7 +713,7 @@ class Session():
                 *api_calls[0].args,
                 **api_calls[0].kwargs
                 )
-            if not resp or len(api_calls) == 1:
+            if (not resp and not continue_on_fail) or len(api_calls) == 1:
                 return [resp]
 
             m_resp: List[Response] = [resp]
@@ -733,7 +748,7 @@ class Session():
 
     # TODO return a BatchResponse object (subclass Response) where OK indicates all OK
     # and method that returns merged output from all resp...
-    def batch_request(self, api_calls: List[BatchRequest],) -> List[Response]:
+    def batch_request(self, api_calls: List[BatchRequest], continue_on_fail: bool = False) -> List[Response]:
         """non async to async wrapper for multiple parallel API calls
 
         First entry is ran alone, if successful the remaining calls
@@ -741,11 +756,13 @@ class Session():
 
         Args:
             api_calls (List[BatchRequest]): List of BatchRequest objects.
+            continue_on_fail (bool, optional): Continue with subsequent requests if first request fails.
+                defaults to False.  Only the first request is validated for success.
 
         Returns:
             List[Response]: List of centralcli.response.Response objects.
         """
-        return asyncio.run(self._batch_request(api_calls))
+        return asyncio.run(self._batch_request(api_calls, continue_on_fail=continue_on_fail))
 
     async def get(self, url, params: dict = {}, headers: dict = None, **kwargs) -> Response:
         f_url = self.auth.central_info["base_url"] + url
